/*
 * Copyright (C) 2001, 2002 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License, 
 * The Sun (tm) Java Advanced Imaging library (JAI), The Sun JIMI library 
 * (or with modified versions of the above that use the same license as the above), 
 * and distribute linked combinations including the two.  You must obey the 
 * GNU General Public License in all respects for all of the code used other than 
 * the above mentioned libraries.  If you modify this file, you may extend this 
 * exception to your version of the file, but you are not obligated to do so.  
 * If you do not wish to do so, delete this exception statement from your version.
 */

package mir.util;

import multex.Failure;
import org.apache.oro.text.regex.*;

/**
 * Simple parser. Can be used to parse a <code>String</code> based using
 * regular epxression.
 */
public class SimpleParser {
  private int position;

  private Perl5Compiler compiler = new Perl5Compiler();
  private Perl5Matcher matcher = new Perl5Matcher();
  private PatternMatcherInput input;


  /**
   * Initialize a new <code>SimpleParser</code>, with <code>someData</code>
   * as the text to parse.
   *
   * @param someData the text to parse
   */

  public SimpleParser(String someData) {
    position = 0;
    input = new PatternMatcherInput(someData);
  }

  /**
   * Parses a regular expression. Uses suppled <code>aMessage</code> parameter
   * in case of an error.
   *
   * @param aRegularExpression  the expression to find at the cursor position
   * @param aMessage the error message to apply if the regular expression couldn't be found
   *
   * @return the text that matched the regular expression
   *
   * @throws mir.util.SimpleParser.UnExpectedTokenExc
   *     if the token wasn't found. <code>aMessage</code> will be used in the
   *     exception's message 
   */
  public String parse(Pattern aRegularExpression, String aMessage) throws UnExpectedTokenExc {
    input.setCurrentOffset(position);
    if ( matcher.contains(input, aRegularExpression) && input.getMatchBeginOffset() == position) {
        position = input.getMatchEndOffset();

        return input.match();
    }
    else {
      throw new UnExpectedTokenExc(aMessage + " at position " +
              position + " in '" + input.toString() + "'");
    }
  }

  /**
   * Attempts to match the data right after the parsing position to
   * a regular expression
   *
   * @throws UnExpectedTokenExc if no match could be found for the given regular
   * expression.
   */
  public String parse(Pattern aRegularExpression) throws UnExpectedTokenExc {
    return parse( aRegularExpression, "No match found for '" + aRegularExpression.getPattern() + "'");
  }

  /**
   * Attempts to skip data at the parsing position matching the supplied regular expression.
   * If no match is found, the method will simply return 
   *
   * @param aRegularExpression the expression to find and skip at the parsing position.
   */

  public void skip(Pattern aRegularExpression) {
      input.setCurrentOffset(position);
    
      if (matcher.contains(input, aRegularExpression) &&
          input.getMatchBeginOffset() == position) {
          position = input.getMatchEndOffset();
      }
  }

  /**
   * Returns <code>true</code> if the data at the parsing position matches the given
   * regular expression.
   */
  public boolean parses(String anExpression) throws MalformedRegularExpressionExc {
    try {
      return parses(compiler.compile(anExpression));
    }
    catch (MalformedPatternException e) {
        throw new MalformedRegularExpressionExc(e);
    }
  }

  /**
   * Tests whether the data at the parsing position matches a given
   * regular expression.
   *
   * @param aRegularExpression the pattern to look for
   * @return returns <code>true</code> if the regular expression is found at the
   *    parsing position
   */
  public boolean parses(Pattern aRegularExpression) {
     input.setCurrentOffset(position);

     return matcher.contains(input, aRegularExpression) && input.getMatchBeginOffset() == position;
  }

  /**
   * Attempts to match the data right after the parsing position to
   * a regular expression
   *
   * @throws SimpleParserExc if no match could be found for the given regular
   * expression.
   */
  public String parse(String anExpression) throws MalformedRegularExpressionExc, UnExpectedTokenExc {
    try {
      return parse(compiler.compile(anExpression));
    }
    catch (MalformedPatternException e) {
        throw new MalformedRegularExpressionExc(e);
    }
  }


  /**
   * Parses text at the current parsing position matching the given <code>anExpression</code>.
   * Will apply the supplied message to an exception if no match could be found.
   */
  public String parse(String anExpression, String aMessage) throws MalformedRegularExpressionExc, UnExpectedTokenExc {
    try {
      return parse(compiler.compile(anExpression), aMessage);
    }
    catch (MalformedPatternException e) {
        throw new MalformedRegularExpressionExc(e);
    }
  }

  /**
   * Skips (i.e. discards) text matching the supplied regular expression
   */

  public void skip(String anExpression) throws MalformedRegularExpressionExc {
    try {
      skip(compiler.compile(anExpression));
    }
    catch (MalformedPatternException e) {
        throw new MalformedRegularExpressionExc(e);
    }
  }

  /**
   * returns <code>true</code> if the parser is at the end of the data
   */
  public boolean isAtEnd() {
    return position >=input.getEndOffset();
  }

  /**
   * Returns the "remaining" data: the data after the current position
   */
  public String remainingData() {
    return input.toString().substring(input.getEndOffset());
  }

  /**
   * Unchecked exception thrown from <code>SimpleParser</code>
   */

  public static class SimpleParserFailure extends Failure {
    public SimpleParserFailure(Throwable aThrowable) {
      super(aThrowable.getMessage(), aThrowable);
    }

    public SimpleParserFailure(String aMessage, Throwable aThrowable) {
      super(aMessage, aThrowable);
    }
  }

  /**
   * Checked exception thrown from <code>SimpleParser</code>
   */
  public static class SimpleParserExc extends Exception {
    protected SimpleParserExc(String aMessage, Throwable aCause) {
      super(aMessage, aCause);
    }

    public SimpleParserExc(Throwable aCause) {
      super(aCause);
    }

    public SimpleParserExc(String aMessage) {
      super(aMessage);
    }
  }

  public static class MalformedRegularExpressionExc extends SimpleParserExc {
    public MalformedRegularExpressionExc(String aMessage, Throwable aCause) {
      super(aMessage, aCause);
    }

    public MalformedRegularExpressionExc(Throwable aCause) {
      super(aCause);
    }
  }

  public static class UnExpectedTokenExc extends SimpleParserExc {
    public UnExpectedTokenExc(String aMessage) {
      super(aMessage);
    }
  }

}